close all; clear all;

[audioM,fs] = audioread('c06ss.wav');
[audioF,fs] = audioread('c05ss.wav');

   % use one channel if stereo
   if size(audioM,2)==2 
      audioM = audioM(:,1);
   end
   if size(audioF,2)==2 
      audioF = audioF(:,1);
   end

T = 1/fs;
N = 512;

%% synthesize window
winAmt = .75;
winLen = floor(winAmt*N);
if(mod(winLen,2)==1)
    winLen = winLen-1;
end
halfWinLen = winLen/2;
win = hamming(winLen);
winF = win(1:halfWinLen);           
winB = win(halfWinLen+1:end);                   %% Split window into front and back halves to properly overlap
jump = N-halfWinLen;                            %% samples to shift for every rep

audioMd = [zeros(halfWinLen,1); audioM];
audioFd = [zeros(halfWinLen,1); audioF];        %% Delay so beginning isn't windowed.  
numreps = 1;
if length(audioFd)>=length(audioMd)
    numreps = floor(length(audioMd)/jump)-1;
else 
    numreps = floor(length(audioFd)/jump)-1;    %% Set number of times to repeat algorithm
end
xW = 0;

%% Buffers to store samples for overlapping
currWinBufi = zeros(halfWinLen,1);
currEBufi = zeros(halfWinLen,1);                
currWinBufO = zeros(halfWinLen,1);
currWinBufO2 = zeros(halfWinLen,1);
currWinBufEi1 = zeros(halfWinLen,1);
currWinBufEi2 = zeros(halfWinLen,1);
currWinBufEo1 = zeros(halfWinLen,1);
currWinBufEo2 = zeros(halfWinLen,1);


%% Vectors to read output signal to
recO = zeros(length(audioMd),1);
recO2 = zeros(length(audioFd),1);
eIout1 = zeros(length(audioMd),1);
eOout1 = zeros(length(audioFd),1);
eIout2 = zeros(length(audioMd),1);
eOout2 = zeros(length(audioFd),1);
reci = zeros(length(audioMd),1);
recF = zeros(length(audioFd),1);
excitationi = zeros(length(audioMd),1);

for i = 1:numreps
    %% MALE TO FEMALE
    xW = audioMd((i-1)*jump+1:(i-1)*jump+N);                    %% Read entire buffer
    xH = xW;                                                    %% Preserve non-windowed portion of buffer
    xH(1:halfWinLen) = xW(1:halfWinLen).* winF;                 %% Window first half
    xH(end-halfWinLen+1:end) = xW(end-halfWinLen+1:end).*winB;  %% Window back half
    
    [coeffs,d] = doPrediction(xH,fs);
    [eOuti,outi,outO,eOutF] = mtfEx(coeffs,xW,xH); 
    
    %% FEMALE TO MALE
    xW2 = audioFd((i-1)*jump+1:(i-1)*jump+N);
    xH2 = xW2;
    xH2(1:halfWinLen) = xW2(1:halfWinLen).* winF;
    xH2(end-halfWinLen+1:end) = xW2(end-halfWinLen+1:end).*winB;
    
    [coeffs2,d2] = doPrediction(xH2,fs);
    [eout2,out2,outO2,eoutF2] = ftmEx(coeffs2,xW2,xH2);
    
    %% Check reconstruction using windowing is correct
    outi(end-halfWinLen+1:end) = outi(end-halfWinLen+1:end) .* winB;
    eOuti(end-halfWinLen+1:end) = eOuti(end-halfWinLen+1:end) .* winB;
    reci((i-1)*jump+1:(i-1)*jump+halfWinLen) = outi(1:halfWinLen) + currWinBufi;
    reci((i-1)*jump+halfWinLen+1:(i-1)*jump+N-halfWinLen) = outi(halfWinLen+1:N-halfWinLen);
    excitationi((i-1)*jump+1:(i-1)*jump+halfWinLen) = eOuti(1:halfWinLen) + currEBufi;
    excitationi((i-1)*jump+halfWinLen+1:(i-1)*jump+N-halfWinLen) = eOuti(halfWinLen+1:N-halfWinLen);
    currWinBufi = outi(end-halfWinLen+1:end);
    currEBufi = eOuti(end-halfWinLen+1:end);
    
    %% MALE TO FEMALE: Write to all output and error buffers 
      recO((i-1)*jump+1:(i-1)*jump+halfWinLen) = outO(1:halfWinLen) + currWinBufO;
      recO((i-1)*jump+halfWinLen+1:(i-1)*jump+N-halfWinLen) = outO(halfWinLen+1:N-halfWinLen);
      currWinBufO = outO(end-halfWinLen+1:end);
      
      eIout1((i-1)*jump+1:(i-1)*jump+halfWinLen) = eOuti(1:halfWinLen) + currWinBufEi1;
      eIout1((i-1)*jump+halfWinLen+1:(i-1)*jump+N-halfWinLen) = eOuti(halfWinLen+1:N-halfWinLen);
      currWinBufEi1 = eOuti(end-halfWinLen+1:end);
      
      eOout1((i-1)*jump+1:(i-1)*jump+halfWinLen) = eOutF(1:halfWinLen) + currWinBufEo1;
      eOout1((i-1)*jump+halfWinLen+1:(i-1)*jump+N-halfWinLen) = eOutF(halfWinLen+1:N-halfWinLen);
      currWinBufEo1 = eOutF(end-halfWinLen+1:end);
      
    %% FEMALE TO MALE: Write to all output and error buffers
      recO2((i-1)*jump+1:(i-1)*jump+halfWinLen) = outO2(1:halfWinLen) + currWinBufO2;
      recO2((i-1)*jump+halfWinLen+1:(i-1)*jump+N-halfWinLen) = outO2(halfWinLen+1:N-halfWinLen);
      currWinBufO2 = outO2(end-halfWinLen+1:end);
      
      eIout2((i-1)*jump+1:(i-1)*jump+halfWinLen) = eout2(1:halfWinLen) + currWinBufEi2;
      eIout2((i-1)*jump+halfWinLen+1:(i-1)*jump+N-halfWinLen) = eout2(halfWinLen+1:N-halfWinLen);
      currWinBufEi2 = eout2(end-halfWinLen+1:end);
      
      eOout2((i-1)*jump+1:(i-1)*jump+halfWinLen) = eoutF2(1:halfWinLen) + currWinBufEo2;
      eOout2((i-1)*jump+halfWinLen+1:(i-1)*jump+N-halfWinLen) = eoutF2(halfWinLen+1:N-halfWinLen);
      currWinBufEo2 = eoutF2(end-halfWinLen+1:end);
end

%% Check Reconstruction
vec = 1:length(audioMd);
err = abs(reci-audioMd);
plot(err);
title('Error between original and reconstructed signal');
figure();
vec = 1:length(audioMd);
plot(vec,audioMd);
title('Original Signal Superimposed over reconstructed signal');
hold on
plot(vec,reci,'r');
hold off
 
sigRMS = rms(audioMd);
errRMS = rms(err);
pctEr = errRMS/sigRMS*100           %% amount of error in reconstruction

%% MtoF
soundsc(recO);
audiowrite('maleToFemale.wav',recO, fs);

%% FtoM
soundsc(recO2);
audiowrite('femaletoMale.wav',recO2, fs);

%% Spectrograms
figure()
m = round(0.025/T);
subplot(2,1,1);
spectrogram(audioMd,win,winLen*winAmt,winLen,fs,'yaxis');
title('Original Male');
subplot(2,1,2);
spectrogram(recO,win,winLen*winAmt,winLen,fs,'yaxis');
title('Converted to Female');

figure()
m = round(0.025/T);
subplot(2,1,1);
spectrogram(audioFd,win,winLen*winAmt,winLen,fs,'yaxis');
title('Original Female');
subplot(2,1,2);
spectrogram(recO2,win,winLen*winAmt,winLen,fs,'yaxis');
title('Converted to Male');

%% Male to Female Plots
start = 30;                           %% Choose sample frame to plot
sampIn = audioMd(start*jump+1:start*jump+winLen);
sampOut = recO(start*jump+1:start*jump+winLen);
errIsamp = eIout1(start*jump+1:start*jump+winLen);
errOsamp = eOout1(start*jump+1:start*jump+winLen);

figure()
[lpOrig,d2] = doPredictionWPlot(sampIn,fs);
title('Linear Prediction for Original Magnitude Response (Male)');
figure()
[lpOut,d2] = doPredictionWPlot(sampOut,fs);
title('Linear Prediction for Converted Magnitude Response (Female)');

figure();
subplot(2,1,1);
plot(sampIn);
axis tight;
grid on;
title('Original Time Domain Signal (Male)');
ylabel('Time');
xlabel('Amplitude');
subplot(2,1,2);
plot(sampOut);
axis tight;
grid on;
title('Converted Time Domain Signal (Female)');
ylabel('Time');
xlabel('Amplitude');

figure();
subplot(2,1,1);
plot(errIsamp);
axis tight;
grid on;
title('Original Error Signal (Male)');
ylabel('Time');
xlabel('Amplitude');
subplot(2,1,2);
plot(errOsamp);
axis tight;
grid on;
title('Converted Error Signal (Female)');
ylabel('Time');
xlabel('Amplitude');

figure();
subplot(2,1,1);
zplane(1,lpOrig);
title('Original Linear Predicton Poles (Male)');
subplot(2,1,2);
zplane(1,lpOut);
title('Converted Linear Prediction Poles (Female)');

%% Female to Male Plots
start = 30;
sampIn = audioFd(start*jump+1:start*jump+winLen);
sampOut = recO2(start*jump+1:start*jump+winLen);
errIsamp = eIout2(start*jump+1:start*jump+winLen);
errOsamp = eOout2(start*jump+1:start*jump+winLen);

figure()
[lpOrig,d2] = doPredictionWPlot(sampIn,fs);
title('Linear Prediction for Original Magnitude Response (Female)');
figure()
[lpOut,d2] = doPredictionWPlot(sampOut,fs);
title('Linear Prediction for Converted Magnitude Response (Male)');

figure();
subplot(2,1,1);
plot(sampIn);
axis tight;
grid on;
title('Original Time Domain Signal (Female)');
ylabel('Time');
xlabel('Amplitude');
subplot(2,1,2);
plot(sampOut);
axis tight;
grid on;
title('Converted Time Domain Signal (Male)');
ylabel('Time');
xlabel('Amplitude');

figure()
subplot(2,1,1);
plot(errIsamp);
axis tight;
grid on;
title('Original Error Signal (Female)');
ylabel('Time');
xlabel('Amplitude');
subplot(2,1,2);
plot(errOsamp);
axis tight;
grid on;
title('Converted Error Signal (Male)');
ylabel('Time');
xlabel('Amplitude');

figure();
subplot(2,1,1);
zplane(1,lpOrig);
title('Original Linear Predicton Poles (Female)');
subplot(2,1,2);
zplane(1,lpOut);
title('Converted Linear Prediction Poles (Male)');


