function [eW,y1,y,eF] = ftmEx(coeffs,xW,xH)
% eW: original error
% y1: reconstructed output
% y: edited output
% eF: output Error 

%% Create Error Signals
eW = filter(coeffs,1,xW);
eH = filter(coeffs,1,xH);

%% Find peaks and shift from Female pitch period to Male pitch period
[aC,l] = xcorr(eH,'coeff');
mu = aC(length(eH)-1);                                      %% mu used to determine if block is pitched or not
if mu>-0.9
    peak = aC(length(xW)+1:end);
    peak2 = 0;
    for k = 1:400
        if(peak(k)>peak2)
            peak2 = peak(k);
            fP = k;                                         %% Calculate pitch period based on largest peaks
        end
    end
    
    nFp = round(1.2*fP);                                    %% Pitch period 1.2 times longer for males
    f = 15;                                                 %% factor of lenience for minpeakDistance
    [pks,lcfr] = findpeaks(xW,'MinPeakDistance',nFp-f);     %% Find peaks in error signal
    dif = abs(fP-nFp);                                      %% Find samples to shift between female and male pitch period
    eF = zeros(length(eW),1);

    for j = 1:length(lcfr)
        if j > 3
            eF(lcfr(j)-dif) = eW(lcfr(j));                  %% Read samples into proper positions based on new pitch period
        else
            eF(lcfr(j)) = eW(lcfr(j));
        end
    end    
end

%% Shift formants
poles = roots(coeffs);
poleAng = angle(poles);
poleMag = abs(poles);
poleAngT = poleAng(poleAng ~= pi & poleAng ~= -1*pi);
poleMagT = poleMag(poleAng ~= pi & poleAng ~= -1*pi);
newPoleAng = poleAngT/1.2;
newPoleAngT = newPoleAng(abs(newPoleAng) <= pi);
newPoleMagT = poleMagT(abs(newPoleAng) <= pi);
newPoles = newPoleMagT.*exp(1i*newPoleAngT);
newCoeffs = poly(newPoles);

%% Synthesis
y = filter(1,newCoeffs,eF);
y1 = filter(1,coeffs,eH);
    
end

